package com.link.core.basic.lock;

import com.link.core.util.PropertyUtil;
import java.lang.reflect.Method;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.aop.support.AopUtils;

public class LockInterceptor implements MethodInterceptor {
   Logger logger = LoggerFactory.getLogger(LockInterceptor.class);
   private final LockHandler lockHandler;

   public LockInterceptor(LockHandler lockHandler) {
      this.lockHandler = lockHandler;
   }

   public Object invoke(MethodInvocation methodInvocation) throws Throwable {
      String globalLockFlag = PropertyUtil.getCommonProperty("globalLockFlag");
      if (!"Y".equals(globalLockFlag)) {
         return methodInvocation.proceed();
      } else {
         Class<?> cls = AopProxyUtils.ultimateTargetClass(methodInvocation.getThis());
         if (cls == null && cls != null) {
            cls = methodInvocation.getThis().getClass();
         }

         if (!cls.equals(methodInvocation.getThis().getClass())) {
            if (this.logger.isDebugEnabled()) {
               this.logger.debug(methodInvocation.getThis().getClass() + "-->" + cls);
            }

            return methodInvocation.proceed();
         } else {
            if (this.logger.isDebugEnabled()) {
               this.logger.debug(methodInvocation.toString());
            }

            Method method = methodInvocation.getMethod();
            if (method.isAnnotationPresent(LinkLock.class)) {
               LinkLock lock = (LinkLock)method.getAnnotation(LinkLock.class);
               if (this.logger.isDebugEnabled()) {
                  this.logger.debug(methodInvocation.getThis().getClass().getName() + "." + method.getName() + "-->@Cache");
               }

               return this.lockHandler.proceed(new LockAopProxy(methodInvocation), lock);
            } else {
               Method specificMethod = AopUtils.getMostSpecificMethod(method, methodInvocation.getThis().getClass());
               if (specificMethod.isAnnotationPresent(LinkLock.class)) {
                  LinkLock lock = (LinkLock)specificMethod.getAnnotation(LinkLock.class);
                  if (this.logger.isDebugEnabled()) {
                     this.logger.debug(methodInvocation.getThis().getClass().getName() + "." + specificMethod.getName() + "-->@Cache");
                  }

                  return this.lockHandler.proceed(new LockAopProxy(methodInvocation), lock);
               } else {
                  return methodInvocation.proceed();
               }
            }
         }
      }
   }
}
